@@ -58,6 +58,7 @@ struct _TerminalScreenPrivate
   char *cooked_title, *cooked_icon_title;
   gboolean icon_title_set;
   char *matched_string;
+  int matched_flavor;
   char **override_command;
   GtkWidget *title_editor;
   GtkWidget *title_entry;
@@ -79,12 +80,20 @@ enum {
   LAST_SIGNAL
 };
 
+enum
+{
+  FLAVOR_AS_IS = 0,
+  FLAVOR_DEFAULT_TO_HTTP,
+  FLAVOR_MAILTO,
+  FLAVOR_SKEY
+};
+
 static void terminal_screen_init        (TerminalScreen      *screen);
 static void terminal_screen_class_init  (TerminalScreenClass *klass);
 static void terminal_screen_finalize    (GObject             *object);
 static void terminal_screen_update_on_realize (GtkWidget      *widget,
                                                TerminalScreen *screen);
-static void     terminal_screen_popup_menu         (GtkWidget      *term,
+static gboolean terminal_screen_popup_menu         (GtkWidget      *term,
                                                     TerminalScreen *screen);
 static gboolean terminal_screen_button_press_event (GtkWidget      *term,
                                                     GdkEventButton *event,
@@ -233,17 +242,20 @@ terminal_screen_init (TerminalScreen *screen)
 #define USERCHARS "-A-Za-z0-9"
 #define PASSCHARS "-A-Za-z0-9,?;.:/!%$^*&~\"#'"
 #define HOSTCHARS "-A-Za-z0-9"
-#define SCHEME    "(news|telnet|nttp|file|http|ftp|https)"
+#define SCHEME    "(news|telnet|nttp|file|https?|ftps?|webcal)"
 #define USER      "[" USERCHARS "]+(:["PASSCHARS "]+)?"
 
   terminal_widget_match_add (screen->priv->term,
-      "((" SCHEME "://(" USER "@)?)|(www|ftp)[" HOSTCHARS "]*\\.)[" HOSTCHARS ".]+(:[0-9]*)?");
-  
+      "\\<(" SCHEME "://(" USER "@)?)[" HOSTCHARS ".]+(:[0-9]+)?"
+      "(/[-A-Za-z0-9_$.+!*(),;:@&=?/~#%]*[^]'.}>) \t\r\n,\\\"])?\\>", FLAVOR_AS_IS);
+
   terminal_widget_match_add (screen->priv->term,
-      "((" SCHEME "://(" USER "@)?)"  "|"  "(www|ftp)[" HOSTCHARS "]*\\.)"
-      "[" HOSTCHARS ".]+(:[0-9]+)?/"
-      "[-A-Za-z0-9_$.+!*(),;:@&=?/~#%]*"
-      "[^]'.}>) \t\r\n,\\\"]");
+      "\\<(www|ftp)[" HOSTCHARS "]*\\.[" HOSTCHARS ".]+(:[0-9]+)?"
+      "(/[-A-Za-z0-9_$.+!*(),;:@&=?/~#%]*[^]'.}>) \t\r\n,\\\"])?\\>", FLAVOR_DEFAULT_TO_HTTP);
+
+  terminal_widget_match_add (screen->priv->term, 
+      
+      "\\<(mailto:)?[a-z0-9]+@[a-z0-9][a-z0-9-]*(\\.[a-z0-9][a-z0-9-]*)+\\>", FLAVOR_MAILTO);
 
   terminal_screen_setup_dnd (screen);
   
@@ -522,7 +534,7 @@ reread_profile (TerminalScreen *screen)
   if (terminal_profile_get_use_skey (screen->priv->profile))
     {
       terminal_widget_skey_match_add (screen->priv->term,
-				      "s/key [0-9]* [-A-Za-z0-9]*");
+				      "s/key [0-9]* [-A-Za-z0-9]*", FLAVOR_SKEY);
     }
   else
     {
@@ -1094,6 +1106,7 @@ terminal_screen_launch_child (TerminalScreen *screen)
 
   err = NULL;
   if (!terminal_widget_fork_command (screen->priv->term,
+                                     terminal_profile_get_login_shell (profile),
                                      terminal_profile_get_update_records (profile),
                                      path,
                                      argv,
@@ -1240,7 +1253,8 @@ secure_keyboard_callback (GtkWidget      *menu_item,
 
 static void
 open_url (TerminalScreen *screen,
-          const char     *orig_url)
+          const char     *orig_url,
+          int             flavor)
 {
   GError *err;
   char *url;
@@ -1248,9 +1262,33 @@ open_url (TerminalScreen *screen,
   g_return_if_fail (orig_url != NULL);
 
   /* this is to handle gnome_url_show reentrancy */
-  url = gnome_vfs_make_uri_from_input (orig_url);
   g_object_ref (G_OBJECT (screen));
   
+  switch (flavor)
+    {
+    case FLAVOR_DEFAULT_TO_HTTP:
+      url = g_strdup_printf ("http:%s", orig_url);
+      break;
+    case FLAVOR_MAILTO:
+      if (strncmp ("mailto:", orig_url, 7) != 0)
+        {
+          url = g_strdup_printf ("mailto:%s", orig_url);
+        }
+      else
+        {
+          url = g_strdup (orig_url);
+        }
+      break;
+    case FLAVOR_AS_IS:
+    case FLAVOR_SKEY:
+      url = g_strdup (orig_url);
+      break;
+    default:
+      url = NULL;
+      g_assert_not_reached ();
+    }
+
+  g_printerr ("(%s)(%s)\n", orig_url, url);
   err = NULL;
   gnome_url_show (url, &err);
 
@@ -1281,7 +1319,7 @@ open_url_callback (GtkWidget      *menu_item,
 {
   if (screen->priv->matched_string)
     {
-      open_url (screen, screen->priv->matched_string);
+      open_url (screen, screen->priv->matched_string, screen->priv->matched_flavor);
       g_free (screen->priv->matched_string);
       screen->priv->matched_string = NULL;
     }
@@ -1411,6 +1449,7 @@ terminal_screen_do_popup (TerminalScreen *screen,
   GList *profiles;
   GList *tmp;
   GSList *group;
+  GtkClipboard *clip;
   
   if (screen->priv->popup_menu)
     gtk_widget_destroy (screen->priv->popup_menu);
@@ -1457,10 +1496,16 @@ terminal_screen_do_popup (TerminalScreen *screen,
   gtk_widget_show (menu_item);
   gtk_menu_shell_append (GTK_MENU_SHELL (screen->priv->popup_menu), menu_item);
 
-  menu_item = append_menuitem (screen->priv->popup_menu,
-                               _("C_lose Tab"),
-                               G_CALLBACK (close_tab_callback),
-                               screen);
+  if (terminal_window_get_screen_count (terminal_screen_get_window (screen)) > 1)
+    menu_item = append_menuitem (screen->priv->popup_menu,
+                                 _("C_lose Tab"),
+                                 G_CALLBACK (close_tab_callback),
+                                 screen);
+  else
+    menu_item = append_menuitem (screen->priv->popup_menu,
+                                 _("_Close Window"),
+                                 G_CALLBACK (close_tab_callback),
+                                 screen);
 
   menu_item = gtk_separator_menu_item_new ();
   gtk_widget_show (menu_item);
@@ -1478,13 +1523,15 @@ terminal_screen_do_popup (TerminalScreen *screen,
                                      GTK_STOCK_PASTE,
                                      G_CALLBACK (paste_callback),
                                      screen);
+  clip = gtk_clipboard_get (GDK_NONE);
+  gtk_widget_set_sensitive (menu_item, gtk_clipboard_wait_is_text_available (clip));
   
   menu_item = gtk_separator_menu_item_new ();
   gtk_widget_show (menu_item);
   gtk_menu_shell_append (GTK_MENU_SHELL (screen->priv->popup_menu), menu_item);
 
   profile_menu = gtk_menu_new ();
-  menu_item = gtk_menu_item_new_with_mnemonic (_("Change _Profile"));
+  menu_item = gtk_menu_item_new_with_mnemonic (_("Change P_rofile"));
 
   gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item),
                              profile_menu);
@@ -1577,11 +1624,12 @@ terminal_screen_do_popup (TerminalScreen *screen,
                   event ? event->time : gtk_get_current_event_time ());
 }
 
-static void
+static gboolean
 terminal_screen_popup_menu (GtkWidget      *term,
                             TerminalScreen *screen)
 {
   terminal_screen_do_popup (screen, NULL);
+  return TRUE;
 }
 
 static gboolean
@@ -1601,7 +1649,8 @@ terminal_screen_button_press_event (GtkWidget      *widget,
   screen->priv->matched_string =
     terminal_widget_check_match (term,
                                  event->x / char_width,
-                                 event->y / char_height);
+                                 event->y / char_height,
+                                 &screen->priv->matched_flavor);
   dingus_button = ((event->button == 1) || (event->button == 2));
 
   if (dingus_button &&
@@ -1612,7 +1661,8 @@ terminal_screen_button_press_event (GtkWidget      *widget,
 
       skey_match = terminal_widget_skey_check_match (term,
 						     event->x / char_width,
-						     event->y / char_height);
+						     event->y / char_height,
+                                                     NULL);
       if (skey_match != NULL)
 	{
 	  terminal_skey_do_popup (screen, GTK_WINDOW (terminal_screen_get_window (screen)), skey_match);
@@ -1628,7 +1678,7 @@ terminal_screen_button_press_event (GtkWidget      *widget,
     {
       gtk_widget_grab_focus (widget);
       
-      open_url (screen, screen->priv->matched_string);
+      open_url (screen, screen->priv->matched_string, screen->priv->matched_flavor);
       g_free (screen->priv->matched_string);
       screen->priv->matched_string = NULL;
       return TRUE; /* don't do anything else such as select with the click */
@@ -1735,10 +1785,6 @@ terminal_screen_get_working_dir (TerminalScreen *screen)
       /* Silently ignore failure here, since we may not be on Linux */
       len = readlink (file, buf, sizeof (buf) - 1);
 
-      /*
-       * If readlink fails, get the current working directory
-       * using the command 'pwdx'. This is specific to Solaris.
-       */
       if (len > 0 && buf[0] == '/')
         {
           buf[len] = '\0';
@@ -1746,55 +1792,25 @@ terminal_screen_get_working_dir (TerminalScreen *screen)
           g_free (screen->priv->working_dir);
           screen->priv->working_dir = g_strdup (buf);
         }
-      else
+      else if (len == 0)
         {
-          char *abs_command = NULL;
-          char *exec_command = NULL;
-          char *output = NULL;
-          int exit_status = 0;
-          
-          abs_command = g_find_program_in_path ("pwdx");
-          if (abs_command == NULL)
-            goto pwdx_out;
+          /* On Solaris, readlink returns an empty string, but the
+           * link can be used as a directory, including as a target
+           * of chdir().
+           */
+          char *cwd;
 
-          exec_command = g_strdup_printf ("%s %d", abs_command, 
-                                          screen->priv->child_pid);
-               
-          if (!g_spawn_command_line_sync (exec_command, &output, 
-                                          NULL, &exit_status, NULL))
-            goto pwdx_out;          
-               
-          if (WIFEXITED (exit_status) &&
-              WEXITSTATUS (exit_status) == 0 &&
-              output) 
+          cwd = g_get_current_dir ();
+          if (cwd != NULL)
             {
-              /*
-               * 'pwdx' output format is:
-               * <pid>:\t<cwd>\n
-               */
-              char *p;
-
-              p = strstr (output, ":");
-              if (p == NULL)
-                goto pwdx_out;
-
-              ++p;
-                   
-              while (*p == ' ' || *p == '\t')
-                ++p;
-              g_strchomp (p);
-                   
-              if (*p == '/')
+              if (chdir (file) == 0)
                 {
                   g_free (screen->priv->working_dir);
-                  screen->priv->working_dir = g_strdup (p);
+                  screen->priv->working_dir = g_get_current_dir ();
+                  chdir (cwd);
                 }
+              g_free (cwd);
             }
-
-        pwdx_out:
-          g_free (abs_command);
-          g_free (exec_command);
-          g_free (output);
         }
       
       g_free (file);
@@ -1902,6 +1918,8 @@ terminal_screen_widget_child_died (GtkWidget      *term,
     case TERMINAL_EXIT_RESTART:
       terminal_screen_launch_child (screen);
       break;
+    case TERMINAL_EXIT_HOLD:
+      break;
     }
 }
 
@@ -1985,7 +2003,6 @@ terminal_screen_edit_title (TerminalScreen *screen,
       
       gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
       gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
-      terminal_util_set_labelled_by (entry, GTK_LABEL (label));
       
       gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
       gtk_box_pack_end (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
